/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Masayuki Nakano <masayuki@d-toybox.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"
#include "domstubs.idl"

%{C++
class nsIPrivateTextRangeList;
struct nsTextEventReply;
struct nsReconversionEventReply;
struct nsQueryCaretRectEventReply;
%}

[ptr] native nsIPrivateTextRangeListPtr(nsIPrivateTextRangeList);
[ptr] native nsTextEventReplyPtr(nsTextEventReply);
[ptr] native nsReconversionEventReplyPtr(nsReconversionEventReply);
[ptr] native nsQueryCaretRectEventReplyPtr(nsQueryCaretRectEventReply);


[scriptable, uuid(ce1c0424-c3c0-44b0-97d6-df12deb19d45)]

interface nsIEditorIMESupport : nsISupports
{


  /**
   * beginComposition(nsTextEventReply* aReply) Handles the start of inline input composition.
   */

  [noscript] void beginComposition(in nsTextEventReplyPtr aReply);

  /**
   * setCompositionString() Sets the inline input composition string.
   * beginComposition must be called prior to this.
   */

  [noscript] void setCompositionString(in DOMString aCompositionString, in nsIPrivateTextRangeListPtr aTextRange, in nsTextEventReplyPtr aReply);

  /**
   * endComposition() Handles the end of inline input composition.
   */

  void  endComposition();

  /**
   * queryComposition()  Get the composition position
   */

  [noscript] void queryComposition(in nsTextEventReplyPtr aReply);

  /**
   * forceCompositionEnd() force the composition end
   */

  void forceCompositionEnd();

  /**
   * getReconversionString()  Get the reconvertion string
   */

  [noscript] void getReconversionString(in nsReconversionEventReplyPtr aReply);

  /**
   * Notify for IME when the editor got focus.
   */

  void notifyIMEOnFocus();

  /**
   * Notify for IME when the editor lost focus.
   */

  void notifyIMEOnBlur();

  /**
   * getQueryCaretRect()  Get the query caret rect
   */

  [noscript] void getQueryCaretRect(in nsQueryCaretRectEventReplyPtr aReply);

  /**
   * Get preferred IME status of current widget.
   */

  [noscript] void getPreferredIMEState(out unsigned long aState);

  /**
   * whether this editor has active IME transaction
   */
  readonly attribute boolean composing;
};

